'use strict';

const passport = require('../passeport-config');
const User = require('../models/User');

const jwt = require('jsonwebtoken');
const secretKey = process.env.SECRET_KEY;
const expiresIn = process.env.EXPIRES_IN;

passport.use(User.createStrategy());

// Serialize user
function createUserResponse(user) {
    return {
        _id: user._id,
        username: user.username,
    };
}

/**
 * Create user
 * This can only be done by the logged in user.
 *
 * body User Created user object (optional)
 * returns User
 **/
exports.createUser = async function (body) {
    const {username, password, isAgent} = body;
    const existingUser = await User.findOne({username});

    if (existingUser) {
        return {status: 400, message: 'Le nom d\'utilisateur existe déjà'};
    }

    const user = await User.register(new User({username: username, isAgent: isAgent}), password);
    const newUser = createUserResponse(user);
    return newUser;
};

// Generate JWT token
function generateAuthToken(user) {
    return jwt.sign({username: user.username}, secretKey, {expiresIn: expiresIn});
}

// Authenticate user
exports.authenticateLocal = function (username, password) {
    return new Promise((resolve, reject) => {
        passport.authenticate('local', (err, user, info) => {
            if (err) {
                reject({status: 500, message: err});
            }
            if (!user) {
                reject({status: 404, message: 'Invalid username or password'});
            }
            const token = generateAuthToken(user);
            resolve({token});
        })({body: {username, password}});
    });
};

// Extract token from request
exports.extractUserFromToken = function (token) {
    return new Promise(async (resolve, reject) => {
        try {
            const decoded = await jwt.verify(token, secretKey);
            // Vérifier la date d'expiration
            const currentTimestamp = Math.floor(Date.now() / 1000);
            if (decoded.exp && decoded.exp < currentTimestamp) {
                reject(new Error('Token expiré.'));
                return;
            }

            const user = await User.findOne({username: decoded.username});

            if (!user) {
                reject(new Error('Utilisateur non trouvé.'));
            } else {
                resolve(decoded);
            }
        } catch (err) {
            reject(err);
        }
    });
};

// Authenticate user
exports.logoutUser = async function (token) {
    try {
        await exports.extractUserFromToken(token);
        return JSON.stringify('Déconnexion réussie');
    } catch (error) {
        throw error;
    }
};
